#ifndef CONNMON_H
#define CONNMON_H

#include <libhnl.h>
#include <libconnmon.h>
#include <curses.h>
#include "common/llist.h"
#include "common/common.h"

#define ATTR(x) (field_attrs[FIELD_ ## x])

enum {
	FIELD_MAIN_HEADER_ACTIVE,
	FIELD_MAIN_HEADER_INACTIVE,
	FIELD_MAIN_HEADER_TOTAL,
	FIELD_MAIN_HEADER_TOP,
	FIELD_MAIN_FOOTER_SLASH,
	FIELD_MAIN_FOOTER_KEY,
	FIELD_MAIN_FOOTER_COLON,
	FIELD_MAIN_FOOTER_DESC,
	FIELD_MAIN_BACKGROUND,
	FIELD_HELP_HEADER,
	FIELD_HELP_FOOTER,
	FIELD_HELP_BACKGROUND,
	FIELD_CONN_HOST_IP,
	FIELD_CONN_HOST_NAME,
	FIELD_CONN_HOST_DOTS,
	FIELD_CONN_HOST_PENDING_IP,
	FIELD_CONN_HOST_PENDING_PARENS,
	FIELD_CONN_PROTO_TCP,
	FIELD_CONN_PROTO_UDP,
	FIELD_CONN_PROTO_ICMP,
	FIELD_CONN_PROTO_IP,
	FIELD_CONN_PORT_NUMBER,
	FIELD_CONN_PORT_NAME,
	FIELD_CONN_PORT_COLON,
	FIELD_CONN_DIRECTION,
	FIELD_CONN_RATE_VALUE,
	FIELD_CONN_RATE_UNAVAILABLE,
	FIELD_CONN_RATE_UNIT,
	FIELD_CONN_ROTATOR,
	FIELD_CONN_TCP_STATE,
	FIELD_CONN_ICMP_TYPE,
	FIELD_CONNLIST_BACKGROUND,
	FIELD_TOTAL_CONN_COUNT,
	FIELD_TOTAL_CONN_TEXT,
	FIELD_TOTAL_BACKGROUND,
	FIELD_MAIN_FOOTER_BACKGROUND,
	FIELD_COUNT,
};

typedef enum {
	CTD_STATE_CREATED,
	CTD_STATE_REMOVED,
	CTD_STATE_OPEN,
	CTD_STATE_CLOSED,
} ConnTrackState;

typedef struct {
	ConnTrack *ct;
	ConnTrackState state;
	uint8_t pos;
	uint8_t rotator;
	uint8_t inactivity;

	char *src_name;
	HNLQuery *src_query;
	bool src_looked;
	char *dst_name;
	HNLQuery *dst_query;
	bool dst_looked;

	const char *src_port;
	const char *dst_port;

	int64_t print_time;
} ConnTrackData;

typedef struct {
	LList *connections;
	Iterator *connection_it;
	WINDOW *window;
	ConnTrackState desired_state;
	uint32_t added;
	int start_pos;
	int height;
	int y;
	bool redraw_in;
	bool redraw_out;
	char *title;
} ConnList;

#define DEFAULT_SLEEP_TIME			300000
#define DEFAULT_INACTIVITY_COUNT	4
#define DEFAULT_HNL_SERVICES		"hosts,adns"
#define DEFAULT_PNL_FILES			"/etc/services"

/* connmon.c */
extern ConnMon *cm;
extern ConnList *oc_list;
extern ConnList *cc_list;
extern uint8_t inactivity_count;
void free_connection(ConnTrackData *data);

/* ui.c */
void connmon_ui_finish(void);
void connmon_ui_initialize(void);
void connmon_ui_resized(void);
void connmon_ui_redraw(void);
bool connmon_ui_read_input(void);
void connmon_ui_connections_updated(void);
void connmon_ui_draw_header(int y, char *title, int count, int start, int height, bool iscur);
void connmon_ui_redisplay_connection(ConnTrackData *data);
void connmon_ui_redraw_key_help(int y, char **key);
extern int field_attrs[];
extern const char *const field_names[];

/* ui_connlist.c */
ConnList *create_connection_list(ConnTrackState desired_state, char *title);
void append_connection(ConnList *cl, ConnTrackData *data);
void clean_connection_list(ConnList *cl);
void redraw_connection_list(ConnList *cl);
void modify_connection_list_bounds(ConnList *cl, int height_mod, int y_mod);
void scroll_connection_list(ConnList *cl, int amount);
void redisplay_connection_safe(ConnList *cl, ConnTrackData *data);
void free_connection_list(ConnList *cl);

/* ui_conn.c */
void clear_line(WINDOW *window, int line);
void display_connection(ConnTrack *ct, WINDOW *window, int y, int x);
void display_totals(ConnMon *cm, WINDOW *window);

/* ui_help.c */
void connmon_ui_help_redraw(void);
void connmon_ui_help_display(void);
bool connmon_ui_help_read_input(void);

/* config.c */
bool parse_color_scheme(const char *filename);

#endif
