#define DONT_DEFINE_BOOL
#include <curses.h>
#include <stdio.h>
#include "common/time64.h"
#include "curses-util.h"
#include "connmon.h"

static char *help_msg[] = {
	"",
	"ConnMon " VERSION,
	"Written by Oskar Liljeblad.",
	"",
	"Copyright (C) 2001 Oskar Liljeblad.",
	"This is free software; see the source for copying conditions.  There is NO",
	"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.",
	"",
	"Report bugs to <oskar@osk.mine.nu>.",
	"",
	"Scrolling keys",
	"  Up         Scroll current pane on line up",
	"  Down       Scroll current pane on line down",
	"  Home       Scroll to first line in current pane",
	"  End        Scroll to last line in current pane",
	"  Page Up    Scroll 12 lines up in current pane",
	"  Page Down  Scroll 12 lines down in current pane",
	"",
	"Miscellaneous keys",
	"  Tab        Switch pane",
	"  R          Move pane separator up one line",
	"  F          Move pane separator down one line",
	"  H          Display this help screen",
	"  Q          Quit program",
	"",
	NULL
};

static char *help_keys[] = {
	"Scroll",		"Up", "Dn", "Home", "End", NULL,
	"Back",			"Q", NULL,
	NULL,
};

static WINDOW *help_window;
static int start_pos;
static int max_start_pos;
static int line_count;
static bool redraw;

static void
create_windows(void)
{
	if (help_window != NULL)
		delwin(help_window);

	help_window = newwin(LINES-3, 0, 2, 0);
	wbkgdset(help_window, ATTR(HELP_BACKGROUND));
	werase(help_window);
	wrefresh(help_window);
	redraw = TRUE;
}

void
connmon_ui_help_redraw(void)
{
	if (redraw) {
		int end_pos = min(start_pos+LINES-3, line_count);
		int c;
		max_start_pos = max(0, line_count-(LINES-3));

		connmon_ui_draw_header(1, "Help", line_count, start_pos, LINES-3, TRUE);
		for (c = start_pos; c < end_pos; c++) {
			mvwprintw(help_window, c-start_pos, 0, help_msg[c]);
			wclrtoeol(help_window);
		}
		wrefresh(help_window);
		connmon_ui_redraw_key_help(LINES-1, help_keys);
		refresh();
		redraw = FALSE;
	}
}

void
connmon_ui_help_display(void)
{
	for (line_count = 0; help_msg[line_count] != NULL; line_count++);

	start_pos = 0;
	help_window = NULL;
	create_windows();
	connmon_ui_help_redraw();
}

bool
connmon_ui_help_read_input(void)
{
	int ch;
	while ((ch = getch()) != ERR) {
		int old_start_pos = start_pos;
	
		if (ch == KEY_UP) {
			start_pos = max(start_pos-1, 0);
		} else if (ch == KEY_DOWN) {
			start_pos = min(start_pos+1, max_start_pos);
		} else if (ch == KEY_NPAGE) {
			start_pos = min(start_pos+(LINES-3)-1, max_start_pos);
		} else if (ch == KEY_PPAGE) {
			start_pos = max(start_pos-(LINES-3)+1, 0);
		} else if (ch == KEY_HOME) {
			start_pos = 0;
		} else if (ch == KEY_END) {
			start_pos = max_start_pos;
		} else if (ch == KEY_RESIZE) {
			create_windows();
		} else if (ch == 'q') {
			return FALSE;
		}

		if (start_pos != old_start_pos)
			redraw = TRUE;
	}
	return TRUE;
}
