#ifndef GCONNMON_H
#define GCONNMON_H

#include <libconnmon.h>
#include <libhnl.h>
#include <libpnl.h>
#include "common/llist.h"
#include "common/common.h"

#define DEFAULT_SLEEP_TIME_MS		300
#define DEFAULT_INACTIVITY_COUNT	4

typedef enum {
	CTD_STATE_CREATED,
	CTD_STATE_REMOVED,
	CTD_STATE_OPEN,
	CTD_STATE_CLOSED,
} ConnTrackState;

typedef struct {
	ConnTrack *ct;
	ConnTrackState state;
	int pos;
	
	char *src_spec;
	char *dst_spec;
	char *src_host_name;
	char *dst_host_name;
	const char *src_port_name;
	const char *dst_port_name;
	HNLQuery *src_host_query;
	HNLQuery *dst_host_query;
	bool src_host_looked;
	bool dst_host_looked;
} ConnTrackData;

typedef struct {
	char *name;
	char *param;
} HNLServiceSpec;

typedef struct {
	char *driver_name;
	char *driver_param;
	char *filter;
	uint32_t poll_delay;
	bool enable_hnl;
	LList *hnl_services;
	bool cache_hnl;
	bool enable_pnl;
	LList *pnl_files;
} ConnMonSettings;

/* gconnmon.c */
void free_connection(ConnTrackData *data);
void activate_settings(ConnMonSettings *settings);
gint timeout_tag;

/* maingui.c */
GtkWidget *create_main_window(void);
void append_connection(ConnTrackData *data);
void refresh_connection_lists(void);
void refresh_connection(ConnTrackData *data);
extern ConnMonSettings cur_settings;
HNLServiceSpec *make_hnl_service_spec(const char *name, const char *param);
void free_hnl_service_spec(HNLServiceSpec *spec);
void reset_everything(void);

/* connspec.c */
char **get_connection_columns(ConnTrackData *data);

/* aboutbox.c */
void show_about_dialog(void);

/* settings.c */
void show_settings_dialog(void);

#endif
