#include <config.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include "common/llist.h"
#include "gconnmon.h"

static LList *open_llist;
static LList *closed_llist;
static GtkWidget *open_clist;
static GtkWidget *closed_clist;
static GtkWidget *statusbar;

static void
exit_program(GtkWidget *widget, gpointer data)
{
	gtk_timeout_remove(timeout_tag);
	gtk_main_quit();
}

static void
show_preferences(GtkWidget *widget, gpointer data)
{
	show_settings_dialog();
}

static void
show_about(GtkWidget *widget, gpointer data)
{
	show_about_dialog();
}

static GtkWidget *
create_connection_list(const char *title, GtkWidget **return_clist)
{
	GtkWidget *frame;
	GtkWidget *scrolledwindow;
	GtkWidget *clist;

	frame = gtk_frame_new(title);
	gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);

	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_add(GTK_CONTAINER(frame), scrolledwindow);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	clist = gtk_clist_new(5);
	gtk_container_set_border_width(GTK_CONTAINER(clist), 2);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), clist);
	gtk_clist_column_titles_show(GTK_CLIST(clist));

	gtk_clist_set_column_width(GTK_CLIST(clist), 0, 30);
	gtk_clist_set_column_width(GTK_CLIST(clist), 1, 170);
	gtk_clist_set_column_width(GTK_CLIST(clist), 2, 170);
	gtk_clist_set_column_width(GTK_CLIST(clist), 3, 60);
	gtk_clist_set_column_width(GTK_CLIST(clist), 4, 70);

	gtk_clist_set_column_widget(GTK_CLIST(clist), 0, gtk_label_new("Proto"));
	gtk_clist_set_column_widget(GTK_CLIST(clist), 1, gtk_label_new("Source"));
	gtk_clist_set_column_widget(GTK_CLIST(clist), 2, gtk_label_new("Destination"));
	gtk_clist_set_column_widget(GTK_CLIST(clist), 3, gtk_label_new("Rate"));
	gtk_clist_set_column_widget(GTK_CLIST(clist), 4, gtk_label_new("State"));

	*return_clist = clist;
	return frame;
}

static GtkWidget *
create_main_menubar(void)
{
	GtkWidget *menubar;
	GtkWidget *menu;
	GtkWidget *item1;
	GtkWidget *item2;

	open_llist = llist_new();
	closed_llist = llist_new();

	menubar = gtk_menu_bar_new();

	item1 = gtk_menu_item_new_with_label("File");
	menu = gtk_menu_new();
	  item2 = gtk_menu_item_new_with_label("Exit");
	  gtk_signal_connect(GTK_OBJECT(item2), "activate", exit_program, NULL);
	  gtk_menu_append(GTK_MENU(menu), item2);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item1), menu);
	gtk_menu_bar_append(GTK_MENU_BAR(menubar), item1);

	item1 = gtk_menu_item_new_with_label("Settings");
	menu = gtk_menu_new();
	  item2 = gtk_menu_item_new_with_label("Preferences...");
	  gtk_signal_connect(GTK_OBJECT(item2), "activate", show_preferences, NULL);
	  gtk_menu_append(GTK_MENU(menu), item2);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item1), menu);
	gtk_menu_bar_append(GTK_MENU_BAR(menubar), item1);

	item1 = gtk_menu_item_new_with_label("Help");
	menu = gtk_menu_new();
	  item2 = gtk_menu_item_new_with_label("About...");
	  gtk_signal_connect(GTK_OBJECT(item2), "activate", show_about, NULL);
	  gtk_menu_append(GTK_MENU(menu), item2);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item1), menu);
	gtk_menu_bar_append(GTK_MENU_BAR(menubar), item1);

	return menubar;
}

void
append_connection(ConnTrackData *data)
{
	if (data->state == CTD_STATE_OPEN) {
		llist_add(open_llist, data);
	}
	else if (data->state == CTD_STATE_CLOSED) {
		llist_add(closed_llist, data);
	}
	data->pos = -1;
}

static void
redraw_clist_connection(GtkWidget *list, ConnTrackData *data, int pos)
{
	char **text = get_connection_columns(data);

	if (data->pos != -1) {
		int c;
		for (c = 0; c < 5; c++) {
			char *old;
			gtk_clist_get_text(GTK_CLIST(list), pos, c, &old);
			if (strcmp(old, text[c]) != 0)
				gtk_clist_set_text(GTK_CLIST(list), pos, c, text[c]);
		}
	}
	else {
		gtk_clist_insert(GTK_CLIST(list), pos, text);
	}

	data->pos = pos;
}

void
refresh_connection(ConnTrackData *data)
{
	if (data->pos != -1) {
		GtkWidget *list;

		list = (data->state == CTD_STATE_OPEN ? open_clist : closed_clist);
		redraw_clist_connection(list, data, data->pos);
	}
}

void
reset_everything(void)
{
	llist_clear(open_llist);
	llist_clear(closed_llist);
	gtk_clist_clear(GTK_CLIST(open_clist));
	gtk_clist_clear(GTK_CLIST(closed_clist));
}

void
refresh_connection_lists(void)
{
	Iterator *it;
	char statusmsg[100];
	int pos;

	pos = 0;
	it = llist_iterator(open_llist);
	while (iterator_has_next(it)) {
		ConnTrackData *data = iterator_next(it);
		if (data->state == CTD_STATE_OPEN) {
			redraw_clist_connection(open_clist, data, pos);
			pos++;
		}
		else {
			gtk_clist_remove(GTK_CLIST(open_clist), pos);
			data->pos = -1;
			iterator_remove(it);
			if (data->state == CTD_STATE_REMOVED)
				free_connection(data);
		}
	}
	iterator_free(it);

	pos = 0;
	it = llist_iterator(closed_llist);
	while (iterator_has_next(it)) {
		ConnTrackData *data = iterator_next(it);
		if (data->state == CTD_STATE_CLOSED) {
			redraw_clist_connection(closed_clist, data, pos);
			pos++;
		}
		else {
			gtk_clist_remove(GTK_CLIST(closed_clist), pos);
			data->pos = -1;
			iterator_remove(it);
			if (data->state == CTD_STATE_REMOVED)
				free_connection(data);
		}
	}
	iterator_free(it);

	gtk_statusbar_pop(GTK_STATUSBAR(statusbar), 1);
	sprintf(statusmsg, "%d connections (%d open, %d closed)",
			llist_size(open_llist)+llist_size(closed_llist),
			llist_size(open_llist),
			llist_size(closed_llist));
	gtk_statusbar_push(GTK_STATUSBAR(statusbar), 1, statusmsg);
}

GtkWidget *
create_main_window(void)
{
	GtkWidget *window;
	GtkWidget *vbox;
	GtkWidget *menubar;
	GtkWidget *vpaned;
	GtkWidget *frame;

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "ConnMon " VERSION);
	gtk_signal_connect(GTK_OBJECT(window), "destroy", exit_program, NULL);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), vbox);

	menubar = create_main_menubar();
	gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);
	
	vpaned = gtk_vpaned_new();
	gtk_box_pack_start(GTK_BOX(vbox), vpaned, TRUE, TRUE, 0);

	frame = create_connection_list("Open Connections", &open_clist);
	gtk_paned_pack1(GTK_PANED(vpaned), frame, FALSE, TRUE);

	frame = create_connection_list("Closed Connections", &closed_clist);
	gtk_paned_pack2(GTK_PANED(vpaned), frame, FALSE, TRUE);

	statusbar = gtk_statusbar_new();
	gtk_box_pack_start(GTK_BOX(vbox), statusbar, FALSE, FALSE, 0);

	gtk_window_set_default_size(GTK_WINDOW(window), 500, 400);
	gtk_widget_show_all(window);

	return window;
}
