#include <config.h>
#include <gtk/gtk.h>
#include "common/memory.h"
#include "gconnmon.h"

static GtkWidget *window;
static GtkWidget *driver_name_combo;
static GtkWidget *driver_param_entry;
static GtkWidget *poll_delay_spinbutton;
static GtkWidget *filter_entry;
static GtkWidget *enable_hnl_checkbutton;
static GtkWidget *cache_hnl_checkbutton;
static GtkWidget *enable_pnl_checkbutton;
static GtkWidget *hnl_services_clist;
static GtkWidget *pnl_files_clist;

static GtkWidget *service_name_combo;
static GtkWidget *service_param_entry;
static GtkWidget *file_name_entry;

static void
initialize_settings_from_gui(ConnMonSettings *s)
{
	int c;

	free(s->driver_name);
	free(s->driver_param);
	free(s->filter);
	llist_iterate(s->hnl_services, (IteratorFunc) free_hnl_service_spec);
	llist_clear(s->hnl_services);
	llist_iterate(s->pnl_files, free);
	llist_clear(s->pnl_files);

	s->driver_name = xstrdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(driver_name_combo)->entry)));
	s->driver_param = xstrdup(gtk_entry_get_text(GTK_ENTRY(driver_param_entry)));
	s->filter = xstrdup(gtk_entry_get_text(GTK_ENTRY(filter_entry)));
	s->poll_delay =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(poll_delay_spinbutton));
	s->enable_hnl = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable_hnl_checkbutton));	
	s->cache_hnl = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cache_hnl_checkbutton));	
	s->enable_pnl = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable_pnl_checkbutton));	

	for (c = 0; c < GTK_CLIST(hnl_services_clist)->rows; c++) {
		char *name;
		char *param;
		gtk_clist_get_text(GTK_CLIST(hnl_services_clist), c, 0, &name);
		gtk_clist_get_text(GTK_CLIST(hnl_services_clist), c, 1, &param);
		llist_add(s->hnl_services, make_hnl_service_spec(name, param));
	}
	for (c = 0; c < GTK_CLIST(pnl_files_clist)->rows; c++) {
		char *file;
		gtk_clist_get_text(GTK_CLIST(pnl_files_clist), c, 0, &file);
		llist_add(s->pnl_files, xstrdup(file));
	}
}

static void
initialize_gui_from_settings(ConnMonSettings *s)
{
	Iterator *it;

	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(driver_name_combo)->entry), s->driver_name);
	gtk_entry_set_text(GTK_ENTRY(driver_param_entry), s->driver_param);
	gtk_entry_set_text(GTK_ENTRY(filter_entry), s->filter);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(poll_delay_spinbutton), s->poll_delay);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable_hnl_checkbutton), s->enable_hnl);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cache_hnl_checkbutton), s->cache_hnl);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable_pnl_checkbutton), s->enable_pnl);

	gtk_clist_clear(GTK_CLIST(hnl_services_clist));
	for (it = llist_iterator(s->hnl_services); iterator_has_next(it); ) {
		HNLServiceSpec *spec = iterator_next(it);
		char *columns[2];

		columns[0] = spec->name;
		columns[1] = spec->param;
		gtk_clist_append(GTK_CLIST(hnl_services_clist), columns);
	}
	iterator_free(it);

	gtk_clist_clear(GTK_CLIST(pnl_files_clist));
	for (it = llist_iterator(s->pnl_files); iterator_has_next(it); ) {
		char *file = iterator_next(it);
		char *columns[1];

		columns[0] = file;
		gtk_clist_append(GTK_CLIST(pnl_files_clist), columns);
	}
	iterator_free(it);
}

static void
hnl_add_clicked(GtkWidget *widget, gpointer data)
{
	char *columns[2];
	/* XXX: check that this is not in list already */
	columns[0] = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(service_name_combo)->entry));
	columns[1] = gtk_entry_get_text(GTK_ENTRY(service_param_entry));
	if (columns[0] != NULL && strcmp(columns[0], "") != 0)
		gtk_clist_append(GTK_CLIST(hnl_services_clist), columns);
}

static void
hnl_remove_clicked(GtkWidget *widget, gpointer data)
{
	if (GTK_CLIST(hnl_services_clist)->selection != NULL) {
		int row = (int) GTK_CLIST(hnl_services_clist)->selection->data;
		if (row >= 0)
			gtk_clist_remove(GTK_CLIST(hnl_services_clist), row);
	}
}

static void
hnl_move_up_clicked(GtkWidget *widget, gpointer data)
{
	if (GTK_CLIST(hnl_services_clist)->selection != NULL) {
		int row = (int) GTK_CLIST(hnl_services_clist)->selection->data;
		if (row > 0)
			gtk_clist_row_move(GTK_CLIST(hnl_services_clist), row, row-1);
	}
}

static void
hnl_move_down_clicked(GtkWidget *widget, gpointer data)
{
	if (GTK_CLIST(hnl_services_clist)->selection != NULL) {
		int row = (int) GTK_CLIST(hnl_services_clist)->selection->data;
		if (row < GTK_CLIST(hnl_services_clist)->rows-1)
			gtk_clist_row_move(GTK_CLIST(hnl_services_clist), row, row+1);
	}
}

static void
pnl_add_clicked(GtkWidget *widget, gpointer data)
{
	char *columns[1];
	/* XXX: check that this is not in list already */
	columns[0] = gtk_entry_get_text(GTK_ENTRY(file_name_entry));
	if (columns[0] != NULL && strcmp(columns[0], "") != 0)
		gtk_clist_append(GTK_CLIST(pnl_files_clist), columns);
}

static void
pnl_remove_clicked(GtkWidget *widget, gpointer data)
{
	if (GTK_CLIST(pnl_files_clist)->selection != NULL) {
		int row = (int) GTK_CLIST(pnl_files_clist)->selection->data;
		if (row >= 0)
			gtk_clist_remove(GTK_CLIST(pnl_files_clist), row);
	}
}

static void
pnl_move_up_clicked(GtkWidget *widget, gpointer data)
{
	if (GTK_CLIST(pnl_files_clist)->selection != NULL) {
		int row = (int) GTK_CLIST(pnl_files_clist)->selection->data;
		if (row > 0)
			gtk_clist_row_move(GTK_CLIST(pnl_files_clist), row, row-1);
	}
}

static void
pnl_move_down_clicked(GtkWidget *widget, gpointer data)
{
	if (GTK_CLIST(pnl_files_clist)->selection != NULL) {
		int row = (int) GTK_CLIST(pnl_files_clist)->selection->data;
		if (row < GTK_CLIST(pnl_files_clist)->rows-1)
			gtk_clist_row_move(GTK_CLIST(pnl_files_clist), row, row+1);
	}
}

static void
pnl_fileselection_ok_clicked(GtkWidget *widget, gpointer data)
{
	GtkWidget *fs = data;
	gtk_entry_set_text(GTK_ENTRY(file_name_entry),
			gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
	gtk_widget_destroy(fs);
}

static void
pnl_select_clicked(GtkWidget *widget, gpointer data)
{
	GtkWidget *fs;

	fs = gtk_file_selection_new("Select file");
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fs)->ok_button),
			"clicked", (GtkSignalFunc) pnl_fileselection_ok_clicked, fs);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button),
			"clicked", (GtkSignalFunc) gtk_widget_destroy, (gpointer) fs);
	gtk_widget_show(fs);
}

static void
settings_ok_clicked(GtkWidget *widget, gpointer data)
{
	initialize_settings_from_gui(&cur_settings);
	activate_settings(&cur_settings);
	gtk_widget_destroy(window);
}

static void
settings_apply_clicked(GtkWidget *widget, gpointer data)
{
	initialize_settings_from_gui(&cur_settings);
	activate_settings(&cur_settings);
}

static void
settings_cancel_clicked(GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy(window);
}

void
show_settings_dialog(void)
{
	GtkWidget *box;
	GtkWidget *notebook;
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *separator;
	GtkObject *spinbutton_adj;
	GtkWidget *buttonbox;
	GtkWidget *button;
	GtkWidget *scrolledwindow;
	GtkWidget *box2;
	GList *driver_names_list = NULL;
	GList *service_names_list = NULL;
	
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_default_size(GTK_WINDOW(window), 440, 340);
	box = gtk_vbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(box), 4);
	gtk_container_add(GTK_CONTAINER(window), box);

	notebook = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(box), notebook, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(notebook), 4);

	/* Begin pane 1 */
	table = gtk_table_new(5, 3, FALSE);
	gtk_container_set_border_width(GTK_CONTAINER(table), 4);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 4);

	label = gtk_label_new("Driver Name");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	label = gtk_label_new("Parameters");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	driver_name_combo = gtk_combo_new();
	gtk_combo_set_value_in_list(GTK_COMBO(driver_name_combo), TRUE, FALSE);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(driver_name_combo)->entry), FALSE);
	gtk_table_attach(GTK_TABLE(table), driver_name_combo, 1, 3, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
	driver_names_list = g_list_append(driver_names_list, "netfilter");
	driver_names_list = g_list_append(driver_names_list, "linux");
	gtk_combo_set_popdown_strings(GTK_COMBO(driver_name_combo), driver_names_list);
	g_list_free(driver_names_list);
	
	driver_param_entry = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), driver_param_entry, 1, 3, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);

	separator = gtk_hseparator_new();
	gtk_table_attach(GTK_TABLE(table), separator, 0, 3, 2, 3, GTK_EXPAND|GTK_FILL, 0, 0, 4);

	label = gtk_label_new("Poll Delay");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, GTK_FILL, 0, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	label = gtk_label_new("Connection Filter");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5, GTK_FILL, 0, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	label = gtk_label_new("ms");
	gtk_table_attach(GTK_TABLE(table), label, 2, 3, 3, 4, GTK_FILL, 0, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	
	spinbutton_adj = gtk_adjustment_new(0, 0, 10000, 10, 100, 100);
	poll_delay_spinbutton = gtk_spin_button_new(GTK_ADJUSTMENT(spinbutton_adj), 1, 0);
	gtk_table_attach(GTK_TABLE(table), poll_delay_spinbutton, 1, 2, 3, 4, GTK_EXPAND|GTK_FILL, 0, 0, 0);

	filter_entry = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), filter_entry, 1, 3, 4, 5, GTK_EXPAND|GTK_FILL, 0, 0, 0);

	label = gtk_label_new("Connection Driver");
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, label);
	/* End pane 1 */

	/* Begin pane 2 */
	box2 = gtk_vbox_new(FALSE, 2);
	gtk_container_set_border_width(GTK_CONTAINER(box2), 4);

	enable_hnl_checkbutton = gtk_check_button_new_with_label("Enable Host Name Lookups?");
	gtk_box_pack_start(GTK_BOX(box2), enable_hnl_checkbutton, FALSE, TRUE, 0);

	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_box_pack_start(GTK_BOX(box2), scrolledwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	hnl_services_clist = gtk_clist_new(2);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), hnl_services_clist);
	gtk_clist_set_column_width(GTK_CLIST(hnl_services_clist), 0, 168);
	gtk_clist_set_column_width(GTK_CLIST(hnl_services_clist), 1, 80);
	gtk_clist_column_titles_show(GTK_CLIST(hnl_services_clist));
	label = gtk_label_new("Service Name");
	gtk_clist_set_column_widget(GTK_CLIST(hnl_services_clist), 0, label);
	label = gtk_label_new("Parameters");
	gtk_clist_set_column_widget(GTK_CLIST(hnl_services_clist), 1, label);

	table = gtk_table_new(2, 2, FALSE);
	gtk_box_pack_start(GTK_BOX(box2), table, FALSE, TRUE, 0);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 4);

	label = gtk_label_new("Service Name");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	label = gtk_label_new("Parameters");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	service_name_combo = gtk_combo_new();
	gtk_combo_set_value_in_list(GTK_COMBO(service_name_combo), TRUE, FALSE);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(service_name_combo)->entry), FALSE);
	gtk_table_attach(GTK_TABLE(table), service_name_combo, 1, 2, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
	service_names_list = g_list_append(service_names_list, "hosts");
	service_names_list = g_list_append(service_names_list, "adns");
	service_names_list = g_list_append(service_names_list, "gethostbyname");
	gtk_combo_set_popdown_strings(GTK_COMBO(service_name_combo), service_names_list);
	g_list_free(service_names_list);

	service_param_entry = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), service_param_entry, 1, 2, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);

	buttonbox = gtk_hbutton_box_new();
	gtk_box_pack_start(GTK_BOX(box2), buttonbox, FALSE, TRUE, 0);
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(buttonbox), 20);

	button = gtk_button_new_with_label("Add");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", hnl_add_clicked, NULL);
	gtk_container_add(GTK_CONTAINER(buttonbox), button);
	button = gtk_button_new_with_label("Remove");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", hnl_remove_clicked, NULL);
	gtk_container_add(GTK_CONTAINER(buttonbox), button);
	button = gtk_button_new_with_label("Move Up");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", hnl_move_up_clicked, NULL);
	gtk_container_add(GTK_CONTAINER(buttonbox), button);
	button = gtk_button_new_with_label("Move Down");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", hnl_move_down_clicked, NULL);
	gtk_container_add(GTK_CONTAINER(buttonbox), button);

	cache_hnl_checkbutton = gtk_check_button_new_with_label("Cache Looked Up Host Names?");
	gtk_box_pack_start(GTK_BOX(box2), cache_hnl_checkbutton, FALSE, FALSE, 0);

	label = gtk_label_new("Host Name Lookup");
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box2, label);
	/* End pane 2 */

	/* Begin pane 3 */
	box2 = gtk_vbox_new(FALSE, 2);
	gtk_container_set_border_width(GTK_CONTAINER(box2), 4);

	enable_pnl_checkbutton = gtk_check_button_new_with_label("Enable Port Name Lookups?");
	gtk_box_pack_start(GTK_BOX(box2), enable_pnl_checkbutton, FALSE, TRUE, 0);

	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_box_pack_start(GTK_BOX(box2), scrolledwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	pnl_files_clist = gtk_clist_new(1);
	gtk_container_add(GTK_CONTAINER(scrolledwindow), pnl_files_clist);
	gtk_clist_set_column_width(GTK_CLIST(pnl_files_clist), 0, 80);
	gtk_clist_column_titles_show(GTK_CLIST(pnl_files_clist));
	label = gtk_label_new("File Name");
	gtk_clist_set_column_widget(GTK_CLIST(pnl_files_clist), 0, label);

	table = gtk_table_new(2, 1, FALSE);
	gtk_box_pack_start(GTK_BOX(box2), table, FALSE, TRUE, 0);
	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
	gtk_table_set_col_spacings(GTK_TABLE(table), 4);

	label = gtk_label_new("File Name");
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	file_name_entry = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), file_name_entry, 1, 2, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);

	button = gtk_button_new_with_label("Select...");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", pnl_select_clicked, NULL);
	gtk_table_attach(GTK_TABLE(table), button, 2, 3, 0, 1, GTK_FILL, 0, 0, 0);

	buttonbox = gtk_hbutton_box_new();
	gtk_box_pack_start(GTK_BOX(box2), buttonbox, FALSE, TRUE, 0);
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(buttonbox), 20);

	button = gtk_button_new_with_label("Add");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", pnl_add_clicked, NULL);
	gtk_container_add(GTK_CONTAINER(buttonbox), button);
	button = gtk_button_new_with_label("Remove");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", pnl_remove_clicked, NULL);
	gtk_container_add(GTK_CONTAINER(buttonbox), button);
	button = gtk_button_new_with_label("Move Up");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", pnl_move_up_clicked, NULL);
	gtk_container_add(GTK_CONTAINER(buttonbox), button);
	button = gtk_button_new_with_label("Move Down");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", pnl_move_down_clicked, NULL);
	gtk_container_add(GTK_CONTAINER(buttonbox), button);

	label = gtk_label_new("Port Name Lookup");
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box2, label);
	/* End pane 3 */

	buttonbox = gtk_hbutton_box_new();
	gtk_box_pack_start(GTK_BOX(box), buttonbox, FALSE, TRUE, 0);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(buttonbox), GTK_BUTTONBOX_SPREAD);

	button = gtk_button_new_with_label("OK");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", settings_ok_clicked, NULL);
	gtk_container_add(GTK_CONTAINER(buttonbox), button);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);

	button = gtk_button_new_with_label("Apply");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", settings_apply_clicked, NULL);
	gtk_container_add(GTK_CONTAINER(buttonbox), button);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);

	button = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", settings_cancel_clicked, NULL);
	gtk_container_add(GTK_CONTAINER(buttonbox), button);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);

	initialize_gui_from_settings(&cur_settings);

	gtk_widget_show_all(window);
}
