#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include "filter.h"

static ConnTuple *tuple;

static bool try_expr(Expr *expr);

static int
compare_field(FieldType field, char *string)
{
	switch (field) {
	case FIELD_SRC_PORT:
		return tuple->ps.tcp_udp.src_port - atoi(string);
	case FIELD_DST_PORT:
		return tuple->ps.tcp_udp.dst_port - atoi(string);
	case FIELD_PROTOCOL:
		return tuple->protocol - atoi(string);
	case FIELD_ICMP_TYPE:
		return tuple->ps.icmp.type - atoi(string);
	case FIELD_ICMP_CODE:
		return tuple->ps.icmp.code - atoi(string);
	case FIELD_ICMP_ID:
		return tuple->ps.icmp.id - atoi(string);

	/* FIXME */
	case FIELD_PORT:
	case FIELD_HOST:
	case FIELD_SRC_HOST:
	case FIELD_DST_HOST:
	default:
		return FALSE;
	}
}

static bool
try_logical_expr(LogicalExpr *expr)
{
	switch (expr->op) {
	case LOGOP_AND:
		return (try_expr(expr->expr1) && try_expr(expr->expr2));
	case LOGOP_OR:
		return (try_expr(expr->expr1) || try_expr(expr->expr2));
	case LOGOP_XOR:
		return (try_expr(expr->expr1) ^ try_expr(expr->expr2));
	default:
		return FALSE;
	}
}

static bool
try_compare_expr(CompareExpr *expr)
{
	int result = compare_field(expr->field, expr->string);

	switch (expr->op) {
	case CMPOP_EQ:
		return (result == 0);
	case CMPOP_NE:
		return (result != 0);
	case CMPOP_LT:
		return (result < 0);
	case CMPOP_GT:
		return (result > 0);
	case CMPOP_LE:
		return (result <= 0);
	case CMPOP_GE:
		return (result >= 0);
	default:
		return FALSE;
	}
}

static bool
try_not_expr(NotExpr *expr)
{
	return !try_expr(expr->expr1);
}

static bool
try_expr(Expr *expr)
{
	switch (expr->type) {
	case EXPR_LOGICAL:
		return try_logical_expr((LogicalExpr *) expr);
	case EXPR_COMPARE:
		return try_compare_expr((CompareExpr *) expr);
	case EXPR_NOT:
		return try_not_expr((NotExpr *) expr);
	default:
		return FALSE;
	}
}

bool
try_expression(Expr *expr, ConnTuple *test_tuple)
{
	tuple = test_tuple;
	return try_expr(expr);
}

void
free_expression(Expr *expr)
{
	if (expr->type == EXPR_LOGICAL) {
		LogicalExpr *lexpr = (LogicalExpr *) expr;
		free_expression(lexpr->expr1);
		free_expression(lexpr->expr2);
	}
	else if (expr->type == EXPR_COMPARE) {
		CompareExpr *cexpr = (CompareExpr *) expr;
		free(cexpr->string);
	}
	else if (expr->type == EXPR_NOT) {
		NotExpr *nexpr = (NotExpr *) expr;
		free(nexpr->expr1);
	}

	free(expr);
}

void
dump_expression(Expr *expr, int indent)
{
	int c;

	if (expr->type == EXPR_LOGICAL) {
		LogicalExpr *lexpr = (LogicalExpr *) expr;
		for (c = 0; c < indent; c++) printf(" "); printf("LOGICAL %d\n", lexpr->op);
		for (c = 0; c < indent; c++) printf(" "); printf("expr1:\n");
		dump_expression(lexpr->expr1, indent+2);
		for (c = 0; c < indent; c++) printf(" "); printf("expr2:\n");
		dump_expression(lexpr->expr2, indent+2);
	}
	else if (expr->type == EXPR_COMPARE) {
		CompareExpr *cexpr = (CompareExpr *) expr;
		for (c = 0; c < indent; c++) printf(" "); printf("COMPARE %d\n", cexpr->op);
		for (c = 0; c < indent; c++) printf(" "); printf("field %d\n", cexpr->field);
		for (c = 0; c < indent; c++) printf(" "); printf("string %s\n", cexpr->string);
	}
	else if (expr->type == EXPR_NOT) {
		NotExpr *nexpr = (NotExpr *) expr;
		for (c = 0; c < indent; c++) printf(" "); printf("NOT\n");
		for (c = 0; c < indent; c++) printf(" "); printf("expr1:\n");
		dump_expression(nexpr->expr1, indent+2);
	}
}
