#ifndef FILTER_H
#define FILTER_H

#include "libconnmon.h"
#include "common/common.h"

typedef enum {
	FIELD_PORT,
	FIELD_SRC_PORT,
	FIELD_DST_PORT,
	FIELD_HOST,
	FIELD_SRC_HOST,
	FIELD_DST_HOST,
	FIELD_PROTOCOL,
	FIELD_ICMP_TYPE,
	FIELD_ICMP_CODE,
	FIELD_ICMP_ID,
} FieldType;

typedef enum {
	LOGOP_AND,
	LOGOP_OR,
	LOGOP_XOR,
	CMPOP_EQ,
	CMPOP_NE,
	CMPOP_LT,
	CMPOP_GT,
	CMPOP_LE,
	CMPOP_GE,
} Operator;

typedef enum {
	EXPR_LOGICAL,
	EXPR_COMPARE,
	EXPR_NOT,
} ExprType;

typedef struct {
	ExprType type;
} Expr;

typedef struct {
	Expr e;
	Operator op;
	Expr *expr1;
	Expr *expr2;
} LogicalExpr;

typedef struct {
	Expr e;
	Operator op;
	FieldType field;
	char *string;
} CompareExpr;

typedef struct {
	Expr e;
	Expr *expr1;
} NotExpr;

/* filter.c */
bool try_expression(Expr *expr, ConnTuple *tuple);
void free_expression(Expr *expr);

/* filter_parse.c */
Expr *parse_expression(const char *string);

#endif
