#ifndef LIBCONNMON_H
#define LIBCONNMON_H

#include <config.h>
#include <stdint.h>
#include "common/common.h"

typedef struct _ConnMon ConnMon;
typedef struct _ConnTrack ConnTrack;
typedef struct _ConnTuple ConnTuple;
typedef struct _ConnStats ConnStats;

typedef void (*ConnTrackHandler)(ConnTrack *track);

#ifndef DEFAULT_CONNMON_DRIVER
#define DEFAULT_CONNMON_DRIVER	"netfilter"
#endif

enum {
	CM_PROTO_IP		= 0,
	CM_PROTO_ICMP	= 1,
	CM_PROTO_TCP	= 6,
	CM_PROTO_UDP	= 17,
};

enum {
	CM_TCP_STATE_NONE,
	CM_TCP_STATE_ESTABLISHED,
	CM_TCP_STATE_SYN_SENT,
	CM_TCP_STATE_SYN_RECV,
	CM_TCP_STATE_FIN_WAIT,
	CM_TCP_STATE_FIN_WAIT2,		/* not used by netfilter, but used by linux */
	CM_TCP_STATE_TIME_WAIT,
	CM_TCP_STATE_CLOSE,
	CM_TCP_STATE_CLOSE_WAIT,
	CM_TCP_STATE_LAST_ACK,
	CM_TCP_STATE_LISTEN,
	CM_TCP_STATE_CLOSING,		/* not used by netfilter, but used by linux */
};

enum {
	CM_CT_UNREPLIED		= 1 << 0,
	CM_CT_ASSURED		= 1 << 1,
	CM_CT_SEEN			= 1 << 2,	/* Set if connection is not new */
};

struct _ConnTuple {
	uint32_t src_ip;
	uint32_t dst_ip;
	union {
		struct {
			uint16_t src_port;
			uint16_t dst_port;
		} tcp_udp;
		struct {
			uint8_t type;
			uint8_t code;
			uint16_t id;
		} icmp;
		uint32_t all;
	} ps;
	uint16_t protocol;
};

struct _ConnStats {
	int64_t time;
	uint64_t rx_packets;
	uint64_t tx_packets;
	uint64_t rx_bytes;
	uint64_t tx_bytes;
};

struct _ConnTrack {
	ConnTuple tuple;

	ConnStats cur_stats;
	ConnStats prev_stats;
	ConnStats first_stats;

	uint32_t timeout;
	uint8_t tcp_state;
	uint8_t flags;
	uint32_t use_count;

	void *data;
};

struct _ConnMon {
	ConnTrackHandler create_handler;
	ConnTrackHandler update_handler;
	ConnTrackHandler delete_handler;

	ConnStats total_cur_stats;
	ConnStats total_prev_stats;
	ConnStats total_first_stats;

	uint32_t conn_count;
};

ConnMon *cm_new(const char *args);
bool cm_read_conns(ConnMon *cm);
void cm_get_conns(ConnMon *cm);
void cm_free(ConnMon *cm);
bool cm_set_filter(ConnMon *cm, const char *filter);

#endif
